/***************************************************************************
 *      written by Uwe Berger; 2006                                        *
 *                 bergeruw@gmx.net                                        *
 *                                                                         *
 *                                                                         *
 *   This programm comes without any warranty !!!                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h> 
#include <fcntl.h>
#include <unistd.h>
#include <termios.h> 
#include <string.h> 

//int fd;
//struct termios tio;


//* ula200 - Steuerzeichen
#define STX  0x02
#define ETX  0x03
#define ENQ  0x05
#define ACK  0x06
#define NAK  0x15
#define DC2  0x12
#define DC3  0x13

//* COM - Schnittstelle
#define DEVICE "/dev/ttyUSB0"
#define BAUDRATE B19200

//* Kommandos
#define COMMAND_CLEAR     "clear"
#define COMMAND_BACKLIGHT "backlight"
#define COMMAND_OUT       "out"
#define COMMAND_RESET     "reset"
#define COMMAND_READKEY   "readkey"

//****************************************
//* Hilfeausgabe
static int prog_help()
{
	printf("Syntax:\n");
	printf("=======\n");
	printf("ula200 <clear>\n");
	printf("ula200 <backlight> <0|1>\n");
	printf("ula200 <out> <x> <y> <text>\n");
	printf("ula200 <readkey> [timeout]\n");
	printf("ula200 <reset>\n");
}

//****************************************
//* ula200 - warten auf ACK
static int ula200_wait_ack (int fd)
{
fd_set input_fdset;
struct timeval tv;
int len;
char buffer[16];

	FD_ZERO(&input_fdset);
	FD_SET(fd, &input_fdset);
	buffer[1] = 0;
	tv.tv_sec = 1;
	tv.tv_usec = 0;
	if (select(fd+1, &input_fdset, NULL, NULL, &tv) == -1)
		perror ("ula200: Fehler bei select() ");
	if (FD_ISSET(fd, &input_fdset))
	{
		if ((len = read(fd, buffer, 16)) == -1)
			perror("ula200: Fehler by read() ");
	}
	if (buffer[1] == ACK) return 0;
	else return 1;
}

//****************************************
//* ula200 - Kommando verpacken und senden
static int ula200_write_command(int fd, unsigned char *data)
{
int i;
int pos = 0;
unsigned char buffer[1024];

	do
	{
	//* ula200 - Kommando-Startbyte
	buffer[pos++] = STX;
	//* Nutzdaten (eventuell umwandeln...)
	for (i = 0;  i < strlen(data); i++)
	{
		switch (data[i]) 
		{
			case STX: buffer[pos++] = ENQ;
				  buffer[pos++] = DC2;
				  break;
			case ETX: buffer[pos++] = ENQ;
				  buffer[pos++] = DC3;
				  break;
			case ENQ: buffer[pos++] = ENQ;
				  buffer[pos++] = NAK;
				  break;
			default:  buffer[pos++] = data[i];
		}
	}
	//* ula200 - Kommando-Endebyte
	buffer[pos++] = ETX;
	//* und ueber Schnittstelle schreiben
	write(fd, buffer, pos);
	
	} while (ula200_wait_ack(fd));
	return(0);
}

//****************************************
//* ula200 - Hintergrundbeleuchtung an/aus
static int ula200_backlight (int fd, unsigned char *on_off)
{
unsigned char data[2];
	
	data[0] = 'h';
	data[1] = on_off[0];
	ula200_write_command(fd, data);
}

//****************************************
//* ula200 - Register zum Schreiben oeffnen
static int ula200_register_write_on (int fd)
{
unsigned char data[3];

	data[0] = 'R';
	data[1] = 'E';
	data[2] = '1';
	ula200_write_command(fd, data);
}

//****************************************
//* ula200 - Register wieder schliessen
static int ula200_register_write_off (int fd)
{
unsigned char data[3];

	data[0] = 'R';
	data[1] = 'E';
	data[2] = '0';
	ula200_write_command(fd, data);
}

//****************************************
//* ula200 - Display loeschen
static int ula200_clear (int fd)
{
unsigned char data[1];

	data[0] = 'l';
	ula200_write_command(fd, data);
}

//****************************************
//* ula200 - Cursor positionieren
static int ula200_c_xy (int fd, int x, int y)
{
unsigned char data[3];

	ula200_register_write_on(fd);
	data[0] = 'R';
	data[1] = '0';
	//* gilt nur fuer Diplays mit 20x4...
	switch (y)
	{
		case 0: data[2] = 128 + x;
			break;
		case 1: data[2] = 192 + x;
			break;
		case 2: data[2] = 148 + x;
			break;
		case 3: data[2] = 212 + x;
			break;
	}
	ula200_write_command(fd, data);
	ula200_register_write_off(fd);
}

//****************************************
//* ula200 - Text ausgeben
static int ula200_out_text (int fd, int x, int y, unsigned char *txt)
{
unsigned char data[80];
int i, len;
	
	len = strlen(txt);
	ula200_c_xy (fd, x, y);	
	data[0] = 's';
	data[1] = len;
	for (i = 0; i < len; i++)
	{
		data[i+2] = txt[i];
	}	
	ula200_write_command(fd, data);
}

//****************************************
//* ula200 - Reset
static int ula200_reset (int fd)
{
unsigned char data[2];

	ula200_register_write_on(fd);
	data[0] = 'R';
	data[1] = 'R';
	ula200_write_command(fd, data);
	ula200_register_write_off(fd);
}

//****************************************
//* ula200 - Readkey
static int ula200_readkey (int fd, int timeout)
{
fd_set input_fdset;
int len;
char buffer[16];
struct timeval tv;

	FD_ZERO(&input_fdset);
	FD_SET(fd, &input_fdset);
	//* mit oder ohne Timeout?
	if (timeout > 0)
	{
		tv.tv_sec = timeout; 
		tv.tv_usec = 0;
		if (select(fd+1, &input_fdset, NULL, NULL, &tv) == -1)
			perror ("ula200: Fehler bei select() ");
	}
	else
	{
		if (select(fd+1, &input_fdset, NULL, NULL, NULL) == -1)
			perror ("ula200: Fehler bei select() ");
	}
	if (FD_ISSET(fd, &input_fdset))
	{
		if ((len = read(fd, buffer, 16)) == -1)
			perror("ula200: Fehler by read() ");
		else
		{
			//* Taste gedrueckt und welche?
			if (buffer[1] == 't')
			{
				if (buffer[2] &  1) printf("t1\n");
				if (buffer[2] &  2) printf("t2\n");
				if (buffer[2] &  4) printf("t3\n");
				if (buffer[2] &  8) printf("t4\n");
				if (buffer[2] & 16) printf("t5\n");
				if (buffer[2] & 32) printf("t6\n");
			}
		}
	}
}

//****************************************
//****************************************
int main(int argc, char *argv[])
{
int fd, i;
struct termios tio;

const char *ula200_env = "ULA200_DEVICE";
char *ula200_device;

	ula200_device = getenv(ula200_env);
	if (ula200_device == NULL)
	{
		ula200_device = DEVICE;
	}

	//* Device oeffnen 
	if ( ( fd = open(ula200_device, O_RDWR| O_NOCTTY | O_NDELAY)) < 0 ) 
	{
		printf("ula200: Fehler beim Oeffnen von %s\n", ula200_device);
		return (1);
	} 
	if ( tcgetattr( fd, &tio ) == -1 )
	{ 
		printf("ula200: Fehler beim Auslesen der Parameter von %s\n", ula200_device);
		return (1); 
	};
	//* COM-Parameter setzen (19200,e,8,1)
	tio.c_cflag = 0;
	tio.c_cflag = BAUDRATE | CS8 | PARENB;
	tio.c_iflag = 0;
	tio.c_oflag = 0;
	tio.c_lflag = 0;
	if(tcsetattr(fd, TCSANOW, &tio) == -1) 
	{
		printf("ula200: Fehler beim Setzen der Schnittstellenparameter\n");
		return (1);
	}	

	//* Befehlsverzweigung (1.Argument ist das Schluesselwort)
	if (argc > 1) 
	{
		if (strcmp(argv[1], COMMAND_CLEAR) == 0)
		{
			ula200_clear(fd);
		}
		else if (strcmp(argv[1], COMMAND_BACKLIGHT) == 0)
		{
			if (argc > 2)
			{
				ula200_backlight(fd, argv[2]);
			}
			else
			{
				printf("ula200 %s -> zu wenig Argumente!\n", COMMAND_BACKLIGHT);
				prog_help();
			}
		}
		else if (strcmp(argv[1], COMMAND_OUT) == 0)
		{
			if (argc > 4)
			{
				ula200_out_text (fd, atoi(argv[2]), atoi(argv[3]), argv[4]);
			}
			else 
			{
				printf("ula200 %s -> zu wenig Argumente!\n", COMMAND_OUT);
				prog_help();
			}
		}
		else if (strcmp(argv[1], COMMAND_RESET) == 0)
		{
			ula200_reset(fd);
		}
		else if (strcmp(argv[1], COMMAND_READKEY) == 0)
		{
			if (argc > 2)
			{
				ula200_readkey(fd, atoi(argv[2]));
			}
			else
			{
				ula200_readkey(fd, 0);
			}
		}
		else
		{
			printf("ula200: unbekannter Befehl!\n");
			prog_help();
		}
	}
	else
	{
		printf("ula200: zu wenig Argumente!\n");
		prog_help();
	}

	//* Device wieder schliessen
	close(fd);
	
	return 0;
}
