#!/usr/bin/tclsh

# Device aus Umgebungsvariable lesen
if {[info exists env(ULA200_DEVICE)]} {
	set device $env(ULA200_DEVICE)
} else {
	# Default-Definition Device
	set device /dev/ttyUSB0
}

# Definition Steuerzeichen
set sz_stx [binary format c 2]
set sz_etx [binary format c 3]
set sz_enq [binary format c 5]
set sz_ack [binary format c 6]
set sz_nak [binary format c 15]
set sz_dc2 [binary format c 12]
set sz_dc3 [binary format c 13]


# Device oeffnen und initialisieren
proc open_port {} {
	global com device
	if {[catch {set com [open $device RDWR] }]==1 } {
	    puts "no device on $device"
	    exit
	} else {
	    fconfigure $com -translation binary
	    fconfigure $com -mode 19200,e,8,1
	    fconfigure $com -buffering none
	    fconfigure $com -blocking 1 
	}
}

# Device schliessen
proc close_port {} {
	global com
	close $com
}


# zu sendendes Byte ggf. konvertieren und senden
proc send_byte {b sz} {
	global sz_stx sz_etx sz_enq sz_ack sz_nak
	global sz_dc2 sz_dc3
	global com
	if {$sz} {
	   send_byte_port $b
	} else {
	   if {$b == $sz_stx} {
	      send_byte_port $sz_enq
	      send_byte_port $sz_dc2
	   } elseif {$b == $sz_etx} {
	      send_byte_port $sz_enq
	      send_byte_port $sz_dc3
	   } elseif {$b == $sz_enq} {
	      send_byte_port $sz_enq
	      send_byte_port $sz_nak
	   } else {
	      send_byte_port $b
	   }
	}
}

# ein Byte ueber Schnittstelle senden
proc send_byte_port {b} {
	global com
	puts -nonewline $com $b
	flush $com
	after 1
}

# Displayregister Schreibzugriff ein/aus
proc display_register_writable {on} {
	global sz_stx sz_etx
	send_byte $sz_stx 1
	send_byte "R" 0
	send_byte "E" 0
	if {$on} {
		send_byte "1" 0
	} else {
		send_byte "0" 0
	}
	send_byte $sz_etx 1
}

# Hintergrundbeleuchtung...
proc cmd_backlight {on} {
	global sz_stx sz_etx bl
	send_byte $sz_stx 1
	send_byte "h" 0
	if {$on} {
		send_byte "1" 0
		set bl 1
	} else {
		send_byte "0" 0
		set bl 0
	}
	send_byte $sz_etx 1
}



# xy-Koord. des Cursors setzen
proc cmd_c_xy {x y} {
	global sz_stx sz_etx
	display_register_writable 1
	send_byte $sz_stx 1
	send_byte "R" 0
	send_byte "0" 0
	switch $y {
		0 {set reg_val [expr 128 + $x]}
		1 {set reg_val [expr 192 + $x]}
		2 {set reg_val [expr 148 + $x]}
		3 {set reg_val [expr 212 + $x]}
	}
	send_byte [binary format c $reg_val] 0
	send_byte $sz_etx 1
	display_register_writable 0
}


# Text an aktueller Cursorposition ausgeben
proc cmd_o_text {x y out_text} {
	global sz_stx sz_etx
	cmd_c_xy $x $y
	send_byte $sz_stx 1
	send_byte "s" 0
	set t ""
	set t $out_text
	# Anzahl Zeichen des Textes senden
	set text_len [string length $t]
	send_byte [binary format c $text_len] 0
	# jedes Zeichen des Textes einzeln senden
	for {set i 0} {$i < $text_len} {incr i 1} {
		send_byte [string index $t $i] 0
	}
	send_byte $sz_etx 1
}



# Display loeschen
proc cmd_clear {} {
	global sz_stx sz_etx
	send_byte $sz_stx 1
	send_byte "l" 0
	send_byte $sz_etx 1
}

# Display reset
proc cmd_clear {} {
	global sz_stx sz_etx
	display_register_writable 1	
	send_byte $sz_stx 1
	send_byte "R" 0
	send_byte "R" 0
	send_byte $sz_etx 1
	display_register_writable 0
}

# Tasten...
proc cmd_readkey {} {
    global com
    while {1==1} {
    if {[read $com 1] == "t"} {
	scan [read $com 1] %c t
	catch {
			if {$t& 1} {puts "t1"}
			if {$t& 2} {puts "t2"}
			if {$t& 4} {puts "t3"}
			if {$t& 8} {puts "t4"}
			if {$t&16} {puts "t5"}
			if {$t&32} {puts "t6"}
		      }
	exit
   }
   }
}

# eigenen (sicheren) TCL-Interpreter oeffnen
set parser [interp create -safe]

# Interpreterdefinitionen...
$parser alias backlight cmd_backlight
$parser alias out cmd_o_text
$parser alias clear cmd_clear
$parser alias reset cmd_reset
$parser alias readkey cmd_readkey


# USB-Port oeffnen...
open_port

# Kommando ausfuehren...
$parser eval $argv

# USB-Port oeffnen...
close_port
