#!/usr/bin/tclsh

# Device aus Umgebungsvariable lesen
if {[info exists env(ULA200_DEVICE)]} {
	set device $env(ULA200_DEVICE)
} else {
	# Default-Definition Device
	set device /dev/ttyUSB0
}

set script_dir [file dirname [info script]]


# Definition Steuerzeichen
set sz_stx [binary format c 2]
set sz_etx [binary format c 3]
set sz_enq [binary format c 5]
set sz_ack [binary format c 6]
set sz_nak [binary format c 15]
set sz_dc2 [binary format c 12]
set sz_dc3 [binary format c 13]

# interne Variable
set bl 0



################################################
#  Aus-/Eingabe-Port  initialisieren/oeffnen   #
################################################

# Device oeffnen und initialisieren
if {[catch {set com [open $device RDWR] }]==1 } {
    puts "no device on $device"
    exit
} else {
    fconfigure $com -translation binary
    fconfigure $com -mode 19200,e,8,1
    fconfigure $com -buffering none
    fconfigure $com -blocking 1
}


################################################
#           ULA200-Ausgabefunktionen           #
################################################

# zu sendendes Byte ggf. konvertieren und senden
proc send_byte {b sz} {
	global sz_stx sz_etx sz_enq sz_ack sz_nak
	global sz_dc2 sz_dc3
	global com
	if {$sz} {
	   send_byte_port $b
	} else {
	   if {$b == $sz_stx} {
	      send_byte_port $sz_enq
	      send_byte_port $sz_dc2
	   } elseif {$b == $sz_etx} {
	      send_byte_port $sz_enq
	      send_byte_port $sz_dc3
	   } elseif {$b == $sz_enq} {
	      send_byte_port $sz_enq
	      send_byte_port $sz_nak
	   } else {
	      send_byte_port $b
	   }
	}
}


# ein Byte ueber Schnittstelle senden
proc send_byte_port {b} {
	global com
	puts -nonewline $com $b
	flush $com
}

# Displayregister Schreibzugriff ein/aus
proc display_register_writable {on} {
	global sz_stx sz_etx
	send_byte $sz_stx 1
	send_byte "R" 0
	send_byte "E" 0
	if {$on} {
		send_byte "1" 0
	} else {
		send_byte "0" 0
	}
	send_byte $sz_etx 1
}


# xy-Koord. des Cursors setzen
proc set_cursor_xy {x y} {
	global sz_stx sz_etx
	display_register_writable 1
	send_byte $sz_stx 1
	send_byte "R" 0
	send_byte "0" 0
	switch $y {
		0 {set reg_val [expr 128 + $x]}
		1 {set reg_val [expr 192 + $x]}
		2 {set reg_val [expr 148 + $x]}
		3 {set reg_val [expr 212 + $x]}
	}
	send_byte [binary format c $reg_val] 0
	send_byte $sz_etx 1
	display_register_writable 0
}


# Text an aktueller Cursorposition ausgeben
proc ___output_text_xy {x y t} {
	global sz_stx sz_etx
	set_cursor_xy $x $y
	display_register_writable 1
	for {set i 0} {$i < [string length $t]} {incr i 1} {
		send_byte $sz_stx 1
		send_byte "R" 0
		send_byte "2" 0
		send_byte [string index $t $i] 0
		send_byte $sz_etx 1
	}
	display_register_writable 0
}

# Text an aktueller Cursorposition ausgeben
proc output_text_xy {x y t} {
	global sz_stx sz_etx
	set_cursor_xy $x $y
	for {set i 0} {$i < [string length $t]} {incr i 1} {
		send_byte $sz_stx 1
		send_byte "c" 0
		send_byte [string index $t $i] 0
		send_byte $sz_etx 1
	}
}

# Hintergrundbeleuchtung ein-/ausschalten...
proc toggle_backlight {} {
	global bl sz_stx sz_etx
	send_byte $sz_stx 1
	send_byte "h" 0
	if {$bl} {
		send_byte "0" 0
		set bl 0
	} else {
		send_byte "1" 0
		set bl 1
	}
	send_byte $sz_etx 1
}


# Display loeschen
proc lcd_clear {} {
	global sz_stx sz_etx
	send_byte $sz_stx 1
	send_byte "l" 0
	send_byte $sz_etx 1
}


################################################
#           ULA200-Tastaturabfrage             #
################################################

# ein Byte einlesen, wenn "t", dann ist im naechsten Byte 
# die Tastenbelegung kodiert...
proc lcd_key_read dev {
    global script_dir
    if {[read $dev 1] == "t"} {
	scan [read $dev 1] %c t
	catch {
			if {$t& 1} {toggle_backlight}
			if {$t& 2} {page_1}
			if {$t& 4} {page_2}
			if {$t& 8} {page_3}
			if {$t&16} {exec_script "$script_dir/exec_t5.sh"}
			if {$t&32} {exec_script "$script_dir/exec_t6.sh"}
		      }
   }
}

# wenn etwas an Schnittstelle anliegt, dann einlesen...
fileevent $com readable [list lcd_key_read $com]


################################################
#           Anwendung                          #
################################################

proc mod {a b} {
	return [lindex [split [expr $a / $b] .] 0]
}


set row0 "01234567890123456789"
set row1 "12345678901234567890"
set row2 "23456789012345678901"
set row3 "34567890123456789012"

set zykl 1000
set zykl_id 0
set lcd_max_x 20

# fuellt eine Zeile auf n Zeichen auf
proc fill_row {row n} {
	while {[string length $row] < $n} {
		set row "$row "
	}
	return $row
}

# die vier Zeilen auf dem Display ausgeben
proc output_rows {} {
	global row0 row1 row2 row3 lcd_max_x
	output_text_xy 0 0 [fill_row $row0 $lcd_max_x]
	output_text_xy 0 1 [fill_row $row1 $lcd_max_x]
	output_text_xy 0 2 [fill_row $row2 $lcd_max_x]
	output_text_xy 0 3 [fill_row $row3 $lcd_max_x]
}


################################################
#           Page_1                             #
################################################
proc page_1 {} {
	global row0 row1 row2 row3 zykl zykl_id
	
	after cancel $zykl_id

	#Datum
	set datum [clock format [clock seconds] -format "%x; %X"]
	# Loadwerte
	set fid [open "/proc/loadavg" "r"] 
	gets $fid line
	close $fid
        set loadavg [concat [lindex $line 0]\
			    [lindex $line 1]\
			    [lindex $line 2]\
			    [lindex $line 3]\
			    [lindex $line 4]]
        # uptime...
	set fid [open "/proc/uptime" "r"] 
	gets $fid line
	close $fid
	# ... einen Integer erzeugen
	set uptime [mod [lindex $line 0] 1]
	# ...Tage
	if {[expr $uptime - 86400] > 0} {
		set uptime_d [mod $uptime 86400]
		set uptime [expr $uptime - [expr $uptime_d * 86400]]
	} else {
		set uptime_d 0
	}
	# ...Stunden
	if {[expr $uptime - 3600] > 0} {
		set uptime_h [mod $uptime 3600]
		set uptime [expr $uptime - [expr $uptime_h * 3600]]
		
	} else {
		set uptime_h 0
	}
	# ... Minuten	
	if {[expr $uptime - 60] > 0} {
		set uptime_m [mod $uptime 60]
	} else {
		set uptime_m 0
	}
	set row0 [format "%s" $datum]
	set row1 [format "uptime: %3.0fd %2.0fh %2.0fm" $uptime_d $uptime_h $uptime_m]
	set row2 [format "Load: %s %s %s" [lindex $loadavg 0]\
					  [lindex $loadavg 1]\
					  [lindex $loadavg 2]]
	set row3 [format "Proz.: %s %s" [lindex $loadavg 3] [lindex $loadavg 4]]
	output_rows
        set zykl_id [after $zykl [list page_1]]
}


################################################
#           Page_2                             #
################################################
set old_cpu_proc_stat {0 0 0 0 0}
set cpu_usage {0 0 0 0 0}
proc page_2 {} {
	global row0 row1 row2 row3 zykl zykl_id old_cpu_proc_stat cpu_usage

	after cancel $zykl_id

	#Datum
	set datum [clock format [clock seconds] -format "%x; %X"]
        # cpu-usage (geht z.Z. nur fuer "letzte CPU"...)
	set fid [open "/proc/stat" "r"] 
	gets $fid line
	while {[lindex $line 0] != "cpu"} {
		gets $fid line
	}
	close $fid
	set user [expr [lindex $line 1] - [lindex $old_cpu_proc_stat 1]]
	set nice [expr [lindex $line 2] - [lindex $old_cpu_proc_stat 2]]
	set syst [expr [lindex $line 3] - [lindex $old_cpu_proc_stat 3]]
	set idle [expr [lindex $line 4] - [lindex $old_cpu_proc_stat 4]]
	set cpu_use [expr $user + $nice + $syst]
	set cpu_all [expr $cpu_use + $idle]
	set user [expr $user * 100.0 / $cpu_all]
	set nice [expr $nice * 100.0 / $cpu_all]
	set syst [expr $syst * 100.0 / $cpu_all]
	set idle [expr $idle * 100.0 / $cpu_all]
	set cpu_use [expr $cpu_use * 100.0 / $cpu_all]
	set cpu_usage [concat $cpu_use $user $nice $syst $idle]
	set old_cpu_proc_stat $line

	set row0 [format "%s" $datum]
	set row1 [format "CPU: %6.2f%%" $cpu_use]
	set row2 [format "U/S: %6.2f%% %6.2f%%" $user $syst]
	set row3 [format "N/I: %6.2f%% %6.2f%%" $nice $idle]

	output_rows
        set zykl_id [after $zykl [list page_2]]
}


################################################
#           Page_3                             #
################################################
proc page_3 {} {
	global row0 row1 row2 row3 zykl zykl_id
	
	after cancel $zykl_id
	
	# Speicherplatz Mem/Swap
	set fid [open "/proc/meminfo" "r"] 
	while {[gets $fid line] >= 0} {
          if {[lindex $line 0] == "MemTotal:"} {set ram_total [lindex $line 1]}
          if {[lindex $line 0] == "MemFree:"} {set ram_free [lindex $line 1]}
          if {[lindex $line 0] == "SwapTotal:"} {set swap_total [lindex $line 1]}
          if {[lindex $line 0] == "SwapFree:"} {set swap_free [lindex $line 1]}
	}
	close $fid
        set ram_usage [expr $ram_total - $ram_free]
        set swap_usage [expr $swap_total - $swap_free]

	set row0 "       Mem     Swap "	
        set row1 [format "total %5.1f MB %5.1f" [expr $ram_total / 1000.0] [expr $swap_total / 1000.0]]
        set row2 [format "usage %5.1f MB %5.1f" [expr $ram_usage / 1000.0] [expr $swap_usage / 1000.0]]
        set row3 [format "free  %5.1f MB %5.1f" [expr $ram_free / 1000.0] [expr $swap_free / 1000.0]]

	output_rows
        set zykl_id [after $zykl [list page_3]]
}

################################################
#           exec_script                        #
################################################
proc exec_script {script_name} {
	global zykl_id
	after cancel $zykl_id
	lcd_clear
	exec $script_name
}




lcd_clear
page_1

# Endlos-Loop...
vwait enter-mainloop
